# Generated by Django 5.2.3 on 2025-12-04 10:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('malware_detection', '0003_malwaredetection_error_code_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='malwaredetection',
            name='file_hash',
            field=models.CharField(blank=True, help_text='hash of the file', max_length=255),
        ),
    ]
