# Generated by Django 5.2.3 on 2025-11-28 14:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('malware_detection', '0002_alter_malwaredetection_parameters'),
    ]

    operations = [
        migrations.AddField(
            model_name='malwaredetection',
            name='error_code',
            field=models.IntegerField(blank=True, help_text='error code for the detection', null=True),
        ),
        migrations.AddField(
            model_name='malwaredetection',
            name='error_msg',
            field=models.TextField(blank=True, help_text='error message for the detection'),
        ),
        migrations.AlterField(
            model_name='malwaredetection',
            name='status',
            field=models.CharField(choices=[('pending', 'Pending'), ('processing', 'Processing'), ('failed', 'Failed')], default='pending', help_text='status of the detection', max_length=255),
        ),
        migrations.AddField(
            model_name='malwaredetection',
            name='backend',
            field=models.TextField(blank=True, help_text='backend used for the detection'),
        ),
    ]
