# Generated by Django 5.2.3 on 2025-08-28 13:56

import lasuite.malware_detection.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('malware_detection', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='malwaredetection',
            name='parameters',
            field=models.JSONField(blank=True, default=dict, encoder=lasuite.malware_detection.models.JsonUUIDEncoder, help_text='parameters for the detection', null=True),
        ),
    ]
